;--------------------------------------------------------------------------
; crt0.s - Generic crt0.s for a Z80
;
; Minimum Z80 System mit Mostek 3801 Multi I/O Chip
; 
;  (C)Christian Julius 10.2014
;  christian.julius@gmx.de
;--------------------------------------------------------------------------

    .module crt0            ; Modul Name fr den Linker
 
    ; Memory Map Adressen der Zeropage (vom Benutzer definierbar)
    ; Definitions

    offset           .equ   0x2000   ; Basisadresse des Codes
    reset_vector     .equ   offset
    stack            .equ   0xffff   ; Stack
    adr_vec_table    .equ   0x2040   ; Mode 2 Vektor Tabelle fr STI Mostek Multi I/O
    adr_nmi          .equ   0x2066   ;  NMI Reset Vektor   
    adr_copyright    .equ   0x2080
    adr_userprog     .equ   0x2100

    ; Globale Funktionen und Variablen fr das C-Programm

    .globl  _main
    .globl  _nmi_vint       ; Funktion des NMI Handlers

    ; Interruptroutinen fr den Mostek MK3801 Multi I/O Baustein
    ; Manual STI Baustein MK3801 Figure 7
    .globl  _int_sti_gpi_0  ; General Purpose Interrupt 0
    .globl  _int_sti_gpi_1  ; General Purpose Interrupt 1
    .globl  _int_sti_gpi_2  ; General Purpose Interrupt 2
    .globl  _int_sti_gpi_3  ; General Purpose Interrupt 3
    .globl  _int_sti_timer_d
    .globl  _int_sti_timer_c
    .globl  _int_sti_gpi_4  ; General Purpose Interrupt 4
    .globl  _int_sti_gpi_5  ; General Purpose Interrupt 5
    .globl  _int_sti_timer_b
    .globl  _int_sti_transmit_error
    .globl  _int_sti_transmit_buffer_empty
    .globl  _int_sti_receive_error
    .globl  _int_sti_receive_buffer_full
    .globl  _int_sti_timer_a
    .globl  _int_sti_gpi_6  ; General Purpose Interrupt 6
    .globl  _int_sti_gpi_7  ; General Purpose Interrupt 7
     
    .globl l__INITIALIZER
    .globl s__INITIALIZER
    .globl s__INITIALIZED

    ; -------------------------------------------------------------

    .area   _HEADER (ABS)   
    ; Reset vector bei Kaltstart und Sprung ins User Programm
    .org    reset_vector
     jp init
     
     ;; Tabelle der RST Vektoren fr Software Interrupts
     .org 0x2008
     ret
     .org 0x2010
     ret
     .org 0x2018
     ret
     .org 0x2020
     ret
     .org 0x2028
     ret
     .org 0x2030
     ret
     ;.org 0x2038
     ;reti
      
  
    ;///////////////////////////////////////////////////////////
    ; Tabelle der Mode 2 Int Vektoren auf die Handler Funktionen im C Modul
    ; Reihenfolge beachten !!! Siehe Manual Mostek 3801, Interrupt Tabelle
    .org adr_vec_table
    .dw (_int_sti_gpi_0)
    .dw (_int_sti_gpi_1)
    .dw (_int_sti_gpi_2)
    .dw (_int_sti_gpi_3)
    .dw (_int_sti_timer_d)
    .dw (_int_sti_timer_c)
    .dw (_int_sti_gpi_4)
    .dw (_int_sti_gpi_5)
    .dw (_int_sti_timer_b)
    .dw (_int_sti_transmit_error)
    .dw (_int_sti_transmit_buffer_empty)
    .dw (_int_sti_receive_error)
    .dw (_int_sti_receive_buffer_full)
    .dw (_int_sti_timer_a)
    .dw (_int_sti_gpi_6)
    .dw (_int_sti_gpi_7)
    
    ;///////////////////////////////////////////////////////////
    ; NMI Interrupt 0x66, muss mit retn abgeschlossen werden
    .org  adr_nmi
    jp _nmi_vint        ; Aufruf Handler im C Modul main.c

    .org adr_copyright
    .asciz "Z80/MK3801 Minimal System (C) 10.2014 by C.Julius"
    
    ;///////////////////////////////////////////////////////////
    ; Anordung der Segmente fuer den Linker, ab hier nur relative
    ; Adressen

      .area  _CODE
      .area  _INITIALIZER
      .area  _HOME
      .area  _GSINIT
      .area  _GSFINAL
      .area  _DATA
      .area  _INITIALIZED
      .area  _BSEG
      .area  _BSS
      .area  _HEAP
    
    ;///////////////////////////////////////////////////////////
    ;; ------- Start des Hauptprogramms nach der Zeropage  -----------
    
    .area _CODE
    init:
    
    ld  sp,#stack   ;; Stack an Spitze des RAM legen
    call gsinit

    ; ###### Hauptprogramm aufrufen #######
    call  _main       
endlos:
    halt
    jr endlos
    
   .area  _GSINIT
 
 gsinit:   
 
    ; Initialisiere globale Variablen im RAM
    ld  bc, #l__INITIALIZER
    ld  a, b
    or  a, c
    jr  Z, weiter
    ld  de, #s__INITIALIZED
    ld  hl, #s__INITIALIZER
    ldir
    
weiter:    
    
    ; // Hier muss ein ret rein  
    .area _GSFINAL
      ret


    
 
    
    
    
    
