/////////////////////////////////////////////////////////
/*
   Minimum Z80 System mit Mostek 3801 Multi I/O Chip
   (C)Christian Julius 10.2014
   christian.julius@gmx.de

   HAL der Hardware und Peripherie

*/
/////////////////////////////////////////////////////////

#include "main.h"

#define TRUE    1
#define FALSE   0


extern __at (0x0040) unsigned int  int_vektor_table[16];  // Lokation der Vektortabelle
extern __at (0x2000) unsigned char ram[RAM_SIZE];      // Bereich des RAM
extern __at (0x0000) unsigned char rom[ROM_SIZE];      // ROM
extern __at (0x0000) unsigned char memory[0xffff];     // Kompletter Speicher
extern __at (0x0080) unsigned char copyright[80];      // Zeiger auf den Copyright String


// Segment Codes der 7-Segmentanzeige, invertiert, 0 - F
const char digits[16]= {0xc0,0xf5,0xa8,0xb0,0x95,
                        0x92,0x82,0xf4,0x80,0x90,
                        0x84,0x83,0xab,0xa1,0x8a,
                        0x8e};

uint8_t rx_buf_empty   = FALSE;    // getchar(): Keine zeichen da
                        
/////////////////////////////////////////////////////////
// Halte CPU an und lasse LED leuchten
// Ausstieg nur durch Reset
void stop_cpu() __naked
{
    __asm
         halt
    __endasm ;

    while (1);
}

// Lsst Interrupts zu
void enable_interrupts()
{
     __asm
        im 2
        ld a,#RAM_START >> 8 & 0xff
        ld i,a
        ei
    __endasm;

}
////////////////////////////////////////////////////////
// Schaltet Interrupts ab
void disable_interrupts()
{
    __asm
        di
    __endasm;
}

////////////////////////////////////////////////////////
// Schaltet die RX Uart ab
void disable_rx()
{
    STI_PVR   = STI_IERA;               // RX Buffer Interrupt ein
    STI_IDR   = STI_IDR  & ~0x10;
    STI_IMRA  = STI_IMRA & ~0x10;       // RX full unmask
    clear_rx_buffer();
}

////////////////////////////////////////////////////////
// Schaltet die RX Uart ein
void enable_rx()
{
    STI_PVR   = STI_IERA;       // RX Buffer Interrupt ein
    STI_IDR   = 0x10;
    STI_IMRA  = 0x10;           // RX full unmask
    clear_rx_buffer();
}

////////////////////////////////////////////////////////
// Zeicheneingabe von der Uart
// kein Timeout!
char getchar()
{
    while ((STI_RSR & 0x80) == 0);
    return(STI_UDR);
}


/////////////////////////////////////////////////////////
// Zeichenausgabe fr printf auf der STI mit 9600 Baud
void putchar(char c)
{
    while ((STI_TSR & 0x80) == 0);  // Warte bis Buffer frei...
    STI_UDR = c;                    // Byte einschreiben

}

/////////////////////////////////////////////////////////
// Zeichenkettenausgabe auf Uart
void printstrng(char *k)
{
    while ((*k) != '\0')
        putchar(*(k++));
}


/////////////////////////////////////////////////////////
// Stellt eine Hexadezimale Zahl auf dem 7 Segment dar
// i = 255 = Display aus

void SetDigit(uint8_t i)
{
        SEG7 = digits[i & 0x0f];
}


/////////////////////////////////////////////////////////
// Setze oder lsche einen GPIO Port
// Input: Port Nr, HIGH oder LOW
void Set_GPIO(uint8_t bitnr, _Bool stat)
{
    register uint8_t foo;

    foo = STI_GPIP;         // Register kann nur geladen werden
     if (stat)               // Bit ein
        foo |= (1<<bitnr);
    else                    // Bit lschen
        foo &= ~(1<<bitnr);

    STI_GPIP = foo;
}


/////////////////////////////////////////////////////////
// Setze oder lsche eine LED (0..7) am Port A
// Input: Led Nr, HIGH oder LOW
void SetLed(uint8_t lednr, _Bool stat)
{
    register uint8_t foo;

    // Read...
    foo = PIO8255_PORT_A;
    // Modify...
    if (stat==1)  // LED ein = Bit LOW
        foo &= ~(1<<lednr);
    else
        foo |= (1<<lednr);
    // Write...
    PIO8255_PORT_A = foo;
}

/////////////////////////////////////////////////////////
// Stelle ein ganzes Byte auf der LED Zeile dar
void SetLedByte(uint8_t t)
{
    PIO8255_PORT_A  = ~t;
}

/////////////////////////////////////////////////////////
// Zeitverzgerung
void delay(uint16_t cnt) {
    register volatile uint16_t k;
    for (k=0;k<cnt;k++);
}


