/////////////////////////////////////////////////////////
/*
   Minimum Z80 System mit Mostek 3801 Multi I/O Chip
   (C)Christian Julius 10.2014
   christian.julius@gmx.de

   Hauptprogramm

*/
/////////////////////////////////////////////////////////

#include <asm/z80/features.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "mcurses.h"
#include "hexedit.h"
#include "main.h"

/* 
 * TODO:
 * hexedit() sollte an der passenden Stelle in main() aufgerufen werden.
 *
 */

////////////////////////////////////////////
//      Globale Variablen
/////////////////////////////////////////////

extern uint8_t rx_buf_full;    // Es sind Zeichen da
extern uint8_t rx_buf_crash;   // WR Zeiger hat RD berholt: Buffer wertlos


// Absolute Adressen des gesamten RAMs (ohne Stack) und ROMs
// Diese Array sind Overlays und bentigen keinen Speicherplatz
__at (0x0040) unsigned int  int_vektor_table[16];    // Lokation der Vektortabelle
__at (0x2000) unsigned char ram[RAM_SIZE];      // Bereich des RAM
__at (0x0000) unsigned char rom[ROM_SIZE];      // ROM
__at (0x0000) unsigned char memory[0xffff];     // Kompletter Speicher
__at (0x0080) unsigned char copyright[80];      // Zeiger auf den Copyright String


// Menu Ausgabe
const char* const menu[] = {
  "\r\n---------------------------------",
  "\r\nZ80 System ist online.\r\n",                 // 0
  "---------------------------------",              // 1
  "\r\n[load].......... .Lade User Program",        // 2
  "\r\n[start]...........Start User Program",       // 3
  "\r\n[clear]...........Loesche Memory",           // 4
  "\r\n[dump]........... Dump Memory",              // 5
  "\r\n[reset]...........Restart System",           // 6
  "\r\n[basic]...........Basic Interpreter",        // 7
  "\r\n---",                                        // 8
  "\r\nDeine Wahl?:"                                // 9
};

/////////////////////////////////////////////////////////
/*
    Initialisierung der gesamten Hardware auf den gewnschten
    Betriebsmodus
*/
/////////////////////////////////////////////////////////

void InitHardware()
{
  
    // 8255: Mode 0: Port A,B,C Output
    PIO8255_CNTRL  = 0x80;      //Mode 0: Port A,B,C Output
    PIO8255_PORT_A = 0xff;
    PIO8255_PORT_B = 0;
    PIO8255_PORT_C = 0;

    // 7-Segment ....
    SEG7 = 0xff;

    // STI konfigurieren fr Uart Ausgabe
    STI_UCR   = 0x88;           // = 10001000  UART Control Register 8N1, DIV16
    STI_RSR   = 0x01;           // RX Status - Enable RX
    STI_TSR   = 0x05;           // TX Status - TX High, TX enabled
    // Timer D...
    STI_PVR   = STI_TDDR;       // Zeiger PVR auf Timer D Data
    STI_IDR   = 0x03;           // Timer D Zeitkonstante
    STI_PVR   = STI_TCDCR;      // Zeiger auf Timer C,D Control Register
    STI_IDR   = 0x89;           // Timer D DIV 4, C = Stop
    // GPIO.....
    STI_PVR   = STI_DDR;        // Zeiger PVR auf indirect 6 (DDR)
    STI_IDR   = 0xff;           // GPIO als Output
    STI_GPIP  = 0x00;           // GPIO auf Null setzen
    // Timer A = endlos Run
 
    // STI_TADR  = 0xb8;        // Reload Wert Timer A, ca 100 HZ
    // STI_TABCR = 0x70;        // Timer A DIV 200, maximale Verzgerung
    STI_PVR   = STI_IERA;       // RX Buffer Interrupt ein
    STI_IDR   = 0x10;
    STI_IMRA  = 0x10;           // RX full unmask

    STI_IPRA  = 0x00;           // Lsche Pending Ints
    STI_PVR   = 0x40;           // Vektor auf Mode 2 Tabelle

    __asm
        im 2
        ld a,#RAM_START >> 8 & 0xff
        ld i,a
        ei
    __endasm;

}

void main()
{
    char i = 0;
    char buf[50];

    sprintf(buf,"Hallo Welt %d",i);
    printf("Hallo %d",i);
  
    InitHardware();
    SetDigit(0x0);
 
    while (1)
    {
        delay(4000);
        while (rx_buf_full)
            rx_read_char();

    }


}



// An das Ende des Codes die Interrupt Routinen dran hngen


