#ifndef main_h
    #define main_h

#define STACK_SIZE  0x00ff
#define RAM_START   0x2000
#define RAM_END     0xFFFF
#define RAM_SIZE    (RAM_END-RAM_START)
#define ROM_START   0x0000
#define ROM_END     0x1fff
#define ROM_SIZE    (ROM_END-ROM_START)


#define  HIGH     1
#define  LOW      0
#define uint8_t     unsigned char
#define uint16_t    unsigned int
#define sint8_t     signed char
#define sint16_t    signed int


// Bitmanipulation
#define sbi(a, b) (a) |= (1 << (b))
#define cbi(a, b) (a) &= ~(1 << (b))

// Alternativ:
#define BitSet(arg,val) ((arg) |= (val))
#define BitClr(arg,val) ((arg) &= ~(val))
#define BitFlp(arg,val) ((arg) ^= (val))
#define BitChk(arg,val) (((arg) & (val)) == (val))
#define BitVal(arg,bits,val) (!(((arg) & (bits)) ^ (val)))

// Prototypen
void nmi_vint(void) __critical __interrupt;
void stop_cpu();
void SetLed(uint8_t , _Bool);
void SetLedByte(uint8_t);
void SetDigit(uint8_t i);
void LED_Rotate(uint8_t, uint8_t);
void printstrng(char *);
void delay(uint16_t);
void clear_rx_buffer();
char rx_read_char();
void disable_rx();

/*
    Externe Hardware am Demuxer A5,A6,A7
    
    Mux 0 : $00:    STI Multi I/O Chip
    Mux 1 : $20:    8255 
    Mux 2 : $40:    ADC 0804 
    Mux 3 : $60     7 Segment 

    I/O Map     0x20..0x2F - STI    
    
    0x00        IDR     Indirect Data Register
    0x01        GPIP    Purpose I/O Data Register
    0x02        IPRB    lnterrupt Pending Register B
    0x03        IPRA    Interrupt Pending Register A .
    0x04        ISRB    Interrupt -Service Register B
    0x05        ISRA    Interrupt in-Service Register A
    0x06        IMRB    lnterrupt Mask Register B
    0x07        IMRA    Interrupt Mask Register A
    0x08        PVR     Pointer,/Vector Register
    0x09        TABCR   Timers A and B Control Register
    0x0A        TBDR    Timer B Data Register
    0x0B        TADR    Timer A Data Register
    0x0C        UCR     USART Control Register
    0x0D        RSR     RX Receiver Status Register
    0x0E        TSR     TX Transmitter Status Register
    0x0F        UDR     USART Data Register 
    
    I/O Map     0x40..0x43 - 8255   
    0x20        PORTA   
    0x21        PORTB
    0x22        PORTC
    0x23        8255_CNTRL
    
    I/O Map     0x60 - ADC 0804
    0x40        ADC0804_READ
    
    I/O Map     0x80 - 7_Segment Anzeige
    0x60        SEG7    
    
    
*/

// Special Register STI Multi I/O Chip
__sfr __at 0x00 STI_IDR;
__sfr __at 0x01 STI_GPIP;
__sfr __at 0x02 STI_IPRB;
__sfr __at 0x03 STI_IPRA;
__sfr __at 0x04 STI_ISRB;
__sfr __at 0x05 STI_ISRA;
__sfr __at 0x06 STI_IMRB;
__sfr __at 0x07 STI_IMRA;
__sfr __at 0x08 STI_PVR;
__sfr __at 0x09 STI_TABCR;
__sfr __at 0x0a STI_TBDR;
__sfr __at 0x0b STI_TADR;
__sfr __at 0x0c STI_UCR;
__sfr __at 0x0d STI_RSR;
__sfr __at 0x0e STI_TSR;
__sfr __at 0x0f STI_UDR;


// Indirekt ber STI_IDR adressbierbare Register des STI
#define     STI_SCR     0       // Sync Character Register
#define     STI_TDDR    1       // Timer D Data Register
#define     STI_TCDR    2       // Timer C Data Register
#define     STI_AER     3       // Active Edge Register
#define     STI_IERB    4       // Interrupt Enable Register B
#define     STI_IERA    5       // Interrupt Enable Register A
#define     STI_DDR     6       // Data Direction Register
#define     STI_TCDCR   7       // Timer C und D Control Register

// PIO 8255 Special I/O
__sfr __at 0x20 PIO8255_PORT_A;
__sfr __at 0x21 PIO8255_PORT_B;
__sfr __at 0x22 PIO8255_PORT_C;
__sfr __at 0x23 PIO8255_CNTRL;

// ADC 0804
__sfr __at 0x40 ADC0804_READ;

// 7b Segment
__sfr __at 0x60 SEG7;

#endif

